% GOAT_CMIP5_FolderContent  searches through a CMIP5 data archive based on the CMIP5 naming method
% Copyright (C) Ori Adam, Sept.2013. Edited Apr.2014
% syntax:
% >> [table, fileaddresses] = GOAT_CMIP5_FolderContent(foldername,varargin)
% for a given variable name (CMIP5 standard name), locate experiments, models, realms, etc.
% varargin INPUT OPTIONS (all options are case insensitive)
% 'Display', 'on' or 'off'    % Show table output in command window. default = on.
% 'variable' (or 'var'),{'varname1','varname2',...}
% 'experiment'(or 'exp'),{'experiment1','experiment2',...}
% 'model'(or 'mod'),{'model1','model2',...}
% 'realm'(or 'rea'),{'realm1','realm2',...}
% 'ensemble' (or 'ens'),{'r1i1p1','r1i1p2',...}
% 'begin'or'year1',1990 (integer, scalar)
% 'end'or'year2',2000 (integer, scalar)
% example1: look for CCSM4 model precipiration ('pr') files, beginning from, 1950 for amip experiment
% [table, fileaddresses] = GOAT_CMIP5_FolderContent(foldername,'var','pr','model','CCSM4','begin',1950)
% example2: look for precipiration ('pr') and surface temperature (tas) in all amip files
% [table, fileaddresses] = GOAT_CMIP5_FolderContent(foldername,'var',{'pr','tas','exp','amip')
% OUTPUT:
% table is a cell array of filtered results (displayed on screen)
% fileaddresses = a list of file addresses of the filtered results
%
% see also GOAT_CMIP5_DistributeCMIP5files, GOAT_CMIP5_CMIP52GOAT

function [table, fileaddresses] = GOAT_CMIP5_FolderContent(foldername,varargin)
%[table, fileaddresses] = GOAT_CMIP5_FolderContent(foldername,varname,Experiment,Realm,Model,year1,year2)

path(path,'Utils') ;
path(path,'Utils/DataManagementCode') ;
disp 'locating CMIP files . . .'
[FoldTree, FileTree] = GOAT_FileFolderTrees(foldername) ;
disp(['Total ',int2str(size(FileTree,1)),' files found. Parsing and filtering Files . . .'])
NF = size(FileTree,1) ;
Table = cell(NF,6) ;
FileAddresses = cell(NF,1) ;
Header(1,1) = {'VarName'} ;
Header(1,2) = {'Experiment'} ;
Header(1,3) = {'Realm'} ;
Header(1,4) = {'Model'} ;
Header(1,5) = {'Ensemble'} ;
Header(1,6) = {'Time Span'} ;
% defaults
varname    = {''} ;
Experiment = {''} ;
Ensemble   = {''} ;
Realm      = {''} ;
Model      = {''} ;
year1      = -10e10 ;
year2      =  10e10 ;
Display = 1 ; 
if NF < 100
    Nmod = 0 ;
else
    Nmod = ceil(NF/100) ;
    GOAT_dispstat('','init') ;
end
if nargin > 1
    for j = 1:2:length(varargin)
        switch lower(varargin{j})
            case {'var','variable'}
                if iscell(varargin{j+1})
                    varname = varargin{j+1} ;
                else
                    varname = varargin(j+1) ;
                end
            case {'experiment','exp'}
                if iscell(varargin{j+1})
                    Experiment = varargin{j+1} ;
                else
                    Experiment = varargin(j+1) ;
                end
            case {'model','mod'}
                if iscell(varargin{j+1})
                    Model = varargin{j+1} ;
                else
                    Model = varargin(j+1) ;
                end
            case {'realm','rea'}
                if iscell(varargin{j+1})
                    Realm = varargin{j+1} ;
                else
                    Realm = varargin(j+1) ;
                end
            case {'ensemble','ens'}
                if iscell(varargin{j+1})
                    Ensemble = varargin{j+1} ;
                else
                    Ensemble = varargin(j+1) ;
                end
            case {'begin','beg','year1'}
                year1 = varargin{j+1} ;
            case {'end','year2'}
                year2 = varargin{j+1} ;
            case 'display'
                if strcmpi(varargin{j+1},'off')
                Display = 0 ;
                end
        end
    end
end


years = ones(length(FileTree),2) ;
FileIdx = ones(length(FileTree),1) ;
q = 0 ;
for i = 1: NF
    P = [] ;
    if mod(i,Nmod) == 0
        GOAT_dispstat(['parsing cmip5 files . . . ',num2str(i/NF*100,'%02.0f'),' %'],'stat')
    end
    FileName = cell2mat(FileTree(i,2)) ;
    [FilePath,Name,Ext] = fileparts(FileName) ;
    try
        if ~strncmp(FileName,'wget',4) && ...
                ~strncmp(FileName,'.',1) && ...
                strcmp(Ext,'.nc')
            P = GOAT_CMIP5_parseName(FileName) ;
            q = q + 1 ;
            Table(q,1) = {P.VarName} ;
            Table(q,2) = {P.ExperimentName} ;
            Table(q,3) = {P.RealmName} ;
            Table(q,4) = {P.ModelName} ;
            Table(q,5) = {P.EnsembleName} ;
            Table(q,6) = {P.TimeSpan} ;
            years(q,1) = P.Year1 ;
            years(q,2) = P.Year2 ;
            FileAddresses(q) = {fullfile(foldername,cell2mat(FileTree(i,1)),cell2mat(FileTree(i,2)))} ; 
        end
    catch
        % nevermind
    end
end
Table = Table(1:q,:) ;
years = years(1:q,:) ;
FileIdx = FileIdx(1:q) ;
FileAddresses = FileAddresses(1:q) ;
if strcmp(varname,'')
    VarFlag = ones(q,1) ;
else
    VarFlag = strcmpi(varname,Table(:,1)) ;
end
if strcmp(Experiment,'')
    ExpFlag = ones(q,1) ;
else
    ExpFlag = strcmpi(Experiment,Table(:,2)) ;
end
if strcmp(Realm,'')
    ReaFlag = ones(q,1) ;
else
    ReaFlag = strcmpi(Realm,Table(:,3)) ;
end
if strcmp(Model,'')
    ModFlag = ones(q,1) ;
else
    ModFlag = strcmpi(Model,Table(:,4)) ;
end
if strcmp(Ensemble,'')
    EnsembleFlag = ones(q,1) ;
else
    EnsembleFlag = strcmpi(Ensemble,Table(:,4)) ;
end
YearFlag = ones(q,1) ;
YearFlag(years(:,2)>str2double(year2) | years(:,1)<str2double(year1)) = 0 ;
Flags = VarFlag.*ExpFlag.*ReaFlag.*ModFlag.*YearFlag.*EnsembleFlag ;

NF = sum(Flags) ;
if NF > 0
    disp(['a total of ',int2str(NF),' matching files found.'])
    disp ' '
    disp ' '
    table = [Header ; Table(Flags==1,:)] ;
    fileaddresses = [{'File Address'} ; FileAddresses(Flags==1,:) ] ; 
    %fileaddresses = cell(sum(Flags)+1,1) ;
    %fileaddresses(1,1) = {'File Address'} ;
    
    col1 = char(table(:,1)) ;
    col2 = char(table(:,2)) ;
    col3 = char(table(:,3)) ;
    col4 = char(table(:,4)) ;
    col5 = char(table(:,5)) ;
    col6 = char(table(:,6)) ;
    sp = '   ' ;
    %disp([col1(1,:),sp,col2(1,:),sp,col3(1,:),sp,col4(1,:),sp,col5(1,:),sp,col6(1,:)]) ;
    if Display
    cprintf(-[.3 0 .8],[col1(1,:),sp,col2(1,:),sp,col3(1,:),sp,col4(1,:),sp,col5(1,:),sp,col6(1,:),'\n']) ;
    end
    if NF < 1000
        for i = 1:sum(Flags)
            %fileaddresses(i+1) = {fullfile(foldername,cell2mat(FileTree(FileIdx(i),1)),cell2mat(FileTree(FileIdx(i),2)))} ;
            if Display
            disp([col1(i+1,:),sp,col2(i+1,:),sp,col3(i+1,:),sp,col4(i+1,:),sp,col5(i+1,:),sp,col6(i+1,:)]) ;
            end
        end
        disp ' '
        disp ' '
    else
%         for i = 1:sum(Flags)
%             fileaddresses(i+1) = {fullfile(foldername,cell2mat(FileTree(FileIdx(i),1)),cell2mat(FileTree(FileIdx(i),2)))} ;
%         end
        if Display
        disp 'Table output too long to be displayed.'
        end
        disp ' '
        disp ' '
    end
else
    disp 'No matching files found.'
    table = [] ;
    fileaddresses = [] ;
end


