% GOAT_CMIP5_parseName: parse the name of CMIP5 output files following Data Reference Syntax (DRS) 1.3.1
%syntax:
% >> P = GOAT_CMIP5_parseName(filename)
% P is a structure containing (where relevant)
% P.VarName        = variable name
% P.RealmName      = realm 
% P.ModelName      = model 
% P.ExperimentName = experiment 
% P.EnsembleName   = ensemble 
% P.TimeSpan       = time span string as it appears in the filename 
% P.TimeResolution = GOAT Calendar matching the data frequency 
% P.Year1/2        = begin/end year 
% P.Month1/2       = first/last month 
% P.Day1/1         = first/last day 
%

function P = GOAT_CMIP5_parseName(filename)

[FilePath,FileName] = fileparts(filename) ;
[startIndex,endIndex] = regexp(FileName,'_') ;
P.VarName    = FileName(1:startIndex(1)-1)             ;
P.RealmName  = FileName(endIndex(1)+1:startIndex(2)-1) ;
P.ModelName  = FileName(endIndex(2)+1:startIndex(3)-1) ;
P.ExperimentName    = FileName(endIndex(3)+1:startIndex(4)-1) ;
[Bidx,Eidx] = regexp(FileName,'r(\d+)i(\d+)p(\d+)') ;
if ~isempty(Bidx)
    %P.EnsembleName    = FileName(endIndex(4)+1:startIndex(5)-1) ;
    P.EnsembleName    = FileName(Bidx:Eidx) ;
    year_regx_idx = 5 ;
else
    P.EnsembleName = 'None'         ;
    year_regx_idx = 4 ;
end
if strcmp(P.RealmName,'fx')
P.TimeSpan = 'Invariant' ; 
else
    try
    P.TimeSpan   = FileName(endIndex(year_regx_idx)+1:end) ;
    catch
    P.TimeSpan   = 'Unrecognized' ;    
    end
end
switch lower(P.RealmName)
    case {'day','cfday'}
        P.TimeResolution = 'Daily' ;
    case {'amon','omon','lmon','limon','oimon','aero','cfmon','cfoff'}
        P.TimeResolution = 'Monthly' ;
    case {'fx','oyr','ayr','oclim','aclim'}
        P.TimeResolution = 'List' ;
    case {'6hrlev','6hrplev'}
        P.TimeResolution = 'DailyX4' ;
    case {'3hr','cf3hr'}
        P.TimeResolution = 'DailyX8' ;
    otherwise
        P.TimeResolution = 'Unknown' ;
end

switch P.TimeResolution
    case {'Daily','DailyX4','DailyX8','Daily365','Daily360'}
        P.Year1  = str2double(FileName(endIndex(year_regx_idx)+1:endIndex(year_regx_idx)+4)) ;
        P.Month1 = str2double(FileName(endIndex(year_regx_idx)+5:endIndex(year_regx_idx)+6)) ;
        if strcmp(FileName(endIndex(year_regx_idx)+7),'-')
        P.Day1   = 1 ; 
        P.Year2  = str2double(FileName(endIndex(year_regx_idx)+8:endIndex(year_regx_idx)+11)) ;
        P.Month2 = str2double(FileName(endIndex(year_regx_idx)+12:endIndex(year_regx_idx)+13)) ;
        P.Day2   = GOAT_DaysInMonth(P.Month2,P.Year2,P.TimeResolution) ;
        else
        P.Day1   = str2double(FileName(endIndex(year_regx_idx)+7:endIndex(year_regx_idx)+8)) ;
        P.Year2  = str2double(FileName(endIndex(year_regx_idx)+10:endIndex(year_regx_idx)+13)) ;
        P.Month2 = str2double(FileName(endIndex(year_regx_idx)+14:endIndex(year_regx_idx)+15)) ;
        P.Day2   = str2double(FileName(endIndex(year_regx_idx)+16:endIndex(year_regx_idx)+17)) ;
        end
    case 'Monthly'
        P.Year1 = str2double(FileName(endIndex(year_regx_idx)+1:endIndex(year_regx_idx)+4)) ;
        P.Month1 = str2double(FileName(endIndex(year_regx_idx)+5:endIndex(year_regx_idx)+6)) ;
        P.Day1 = 0 ;
        P.Year2 = str2double(FileName(endIndex(year_regx_idx)+8:endIndex(year_regx_idx)+11)) ;
        P.Month2 = str2double(FileName(endIndex(year_regx_idx)+12:endIndex(year_regx_idx)+13)) ;
        P.Day2 = 0 ; 
%         switch P.Month2
%             case {1,3,5,7,8,10,12}
%                 P.Day2 = 31 ;
%             case 2
%                 P.Day2 = 28 ;
%             otherwise
%                 P.Day2 = 30 ;
%         end
    otherwise
        [P.Year1,P.Month1,P.Day1,P.Year2,P.Month2,P.Day2] = deal(0,0,0,0,0,0) ; 
end
