% Produce an ocean mask for prescribed latitude and longitude vectors 
% The function performs the following:
% 1. Loads a 10' resolution (2160 x 1080) global water percentage map (source: NASA:   http://www-surf.larc.nasa.gov/surf/pages/water.html )
% 2. Interpolates to prescribed latitude and longitude
% 3. Sets land areas as areas below threshold water content (default = 10%)
% 4. ocean_mask = 1 for ocean and 0 for land at the prescribed lon-lat map
% syntax: 
% >> ocean_mask = GOAT_ocean_mask(LON,LAT,varargin) 
% LON and LAT are longitude (degrees) and latitude (degrees) vectors, respectively
% varargin pairs: 
% 1. 'fig'   ,'on'               : display stages of interpolation (default = 'off')
% 2. 'nc'    ,'nc_file_address' or 'default' : produce nc file with  'ocean_mask' as variable and save it to nc_file_address or if 'default', it will be saved at GOAT/Export/ocean_mask/ocean_mask.nc'
% 3. 'cutoff',value (in %)       : a cutoff value of water percentage below which mask = land (default = 10%)
% 4. 'lakes', 'remove' or 'keep' : default = 'keep'. 'remove' = exclude lakes from ocean mask
% 4. 'lakes', box                : remove lakes from ocean mask where lakes are either single gripdpoint ocean points or located within the boxes defined by box.top, box.bottom, box.left, box.right 
% example:
% >> ocean_mask = GOAT_ocean_mask(0:359,-90:90,'fig','on','nc','ocean_mask.nc','cutoff',15)
% will 
% a. produce an ocean_mask variable with longitude = 0:359 and latitude = -90:90 
% b. set land areas (ocean_mask = 0) as grid points where water percentage <=15%
% c. display a figure with the original water percentage, the interpolated water percentage and ocean mask 
% d. save the variable 'ocean_mask' in the NetCDF file 'ocean_mask.nc'
% See below for default lake boxes (lines 99-102)


function ocean_mask = GOAT_ocean_mask(LON,LAT,varargin)
cutoff    = 10 ; % default
FigFlag   = 0  ; % default
ncFlag    = 0  ; % default
LakesFlag = 0  ; % default 
ncAddress = fullfile(fileparts(fileparts(which('GOAT.m'))),'Export','ocean_mask','ocean_mask.nc') ;
disp 'Loading ''water_percentage_map'' . . .'
load('water_percentage_map.mat') ;
if nargin > 2
    for i = 1:length(varargin)/2
        switch lower(varargin{i*2-1})
            case 'fig'
                if strcmpi(varargin{i*2},'on')
                    FigFlag = 1 ;
                end
            case 'nc'
                ncFlag = 1 ;
                if ~strcmpi(varargin{i*2},'default')
                ncAddress = varargin{i*2} ;
                end
            case 'cutoff'
                cutoff = varargin{i*2} ;
            case 'lakes'
                if isstr(varargin{i*2})
                    if strcmpi(varargin{i*2},'remove') || strcmpi(varargin{i*2},'off')
                        LakesFlag = 1 ;
                    end
                elseif isstruct(varargin{i*2})
                    LakesFlag = 2           ; % 
                    box = varargin{i*2}     ;
                    box_west   = box.west   ;
                    box_east   = box.east   ;
                    box_top    = box.top    ;
                    box_bottom = box.bottom ;
                end
        end
    end
end
if ~exist(fileparts(ncAddress),'dir') 
    mkdir(fileparts(ncAddress)) ; 
end
% reshape LON and LAT to be column vectors
if size(LON,2) > size(LON,1) ;
    LON = LON' ;
end
if size(LAT,2) > size(LAT,1) ;
    LAT = LAT' ;
end
if FigFlag
    scrsz      = get(0,'ScreenSize') ;
    disp 'Plotting original high resolution water percentage map (may take a while) . . .'
    figure('position',[scrsz(3) scrsz(4) scrsz(3) scrsz(4)].*[.25 .05 .5 .85]) ;
    subplot 311
    contourf([lon - 360 , lon , lon + 360],lat,[wc;wc;wc]') ;
    xlim([min(LON) max(LON)])
    colormap(gray) ;
    colorbar ;
    xlabel('Longitude','fontsize',14) ;
    ylabel('Latitude','fontsize',14) ;
    set(gca,'linewidth',1.5,'box','off','tickdir','out','fontsize',12)
    title('  Water Percentage  ','fontsize',18)    
end
disp 'Interpolating water percentage mask . . .' ;
water_percentage_I = interp2([lon-360,lon,lon+360],lat',[wc;wc;wc]',LON',LAT)' ;
if FigFlag
    subplot 312
    contourf(LON,LAT,water_percentage_I') ;
    colormap(gray) ;
    colorbar ;
    xlabel('Longitude','fontsize',14) ;
    ylabel('Latitude','fontsize',14) ;
    set(gca,'linewidth',1.5,'box','off','tickdir','out','fontsize',12)
    title('  Interpolated Water Percentage  ','fontsize',18)
end
ocean_mask = ones(size(water_percentage_I)) ;
ocean_mask(water_percentage_I<=cutoff) = 0 ;
if LakesFlag
    if LakesFlag == 1
    box_west   = [9 , 284, 306  , 354, 14  , 59 , 317, 89 , 109, 235, 232] ;
    box_east   = [20, 301, 312  , 360, 36.5, 106, 319, 127, 119, 237, 234] ;
    box_top    = [15, 3  , -18  , 42 , 4   , 57 , -8 , 70 , 31 , 56 , 66 ] ;
    box_bottom = [9 , -8 , -23.5, 10 , -4  , 29 , -10, 67 , 27 , 54 , 64 ] ;
    end
    for j = 1:length(box_west)
        ocean_mask(LON>=box_west(j) & LON <= box_east(j),LAT<=box_top(j) & LAT >= box_bottom(j)) = 0 ;
    end
    % Island rank = number of adjacent ocean grid points, surrounded by land. Island rank = 1 mean a single ocean grid point, surrounded by land. 
    Island_rank = 3 ; 
    %ocean_mask0 = ocean_mask ; 
    ocean_mask0 = [ocean_mask(end-Island_rank-1:end,:) , ocean_mask , ocean_mask(1:Island_rank+2:end,:)] ; 
    for r = 1:Island_rank
      for i = 1:size(ocean_mask0,1) - r - 1
        for j = 1:size(ocean_mask0,2) - r - 1
            mini_mask = ocean_mask(i:i+r+1,j:j+r+1) ;
            % only if completely encircled by land
            if (sum(ocean_mask(i:i+r+1,j)) + sum(ocean_mask(i:i+r+1,j+r+1)) + sum(ocean_mask(i,j+1:j+1+r)) + sum(ocean_mask(i+r+1,j+1:j+2))) == 12
            if sum(mini_mask(:))/16 < 0.15
                ocean_mask(i:i+3,j:j+3) = 0 ;
            end
            end
        end
      end
    end
    % remove single gridpoint islands
    ocean_mask0 = ocean_mask ; 
    ocean_mask0(2:end-1,2:end-1) = (ocean_mask(2:end-1,2:end-1) + ocean_mask(1:end-2,2:end-1) + ocean_mask(3:end,2:end-1) + ocean_mask(2:end-1,1:end-2) + ocean_mask(1:end-2,1:end-2) + ocean_mask(3:end,1:end-2) + ocean_mask(2:end-1,3:end) + ocean_mask(1:end-2,3:end) + ocean_mask(3:end,3:end) ) / 9  ; 
    ocean_mask0(1,2:end-1)       = (ocean_mask(1,2:end-1)       + ocean_mask(end,2:end-1)     + ocean_mask(2,2:end-1)     + ocean_mask(1,1:end-2)       + ocean_mask(end,1:end-2)     + ocean_mask(2,1:end-2)     + ocean_mask(1,3:end)       + ocean_mask(end,3:end)     + ocean_mask(2,3:end)     ) / 9  ; 
    ocean_mask0(end,2:end-1)     = (ocean_mask(end,2:end-1)     + ocean_mask(end-1,2:end-1)   + ocean_mask(1,2:end-1)     + ocean_mask(end,1:end-2)     + ocean_mask(end-1,1:end-2)   + ocean_mask(1,1:end-2)     + ocean_mask(end,3:end)     + ocean_mask(end-1,3:end)   + ocean_mask(1,3:end)     ) / 9  ; 
    ocean_mask(ocean_mask0<0.12) = 0 ;
    % remove double gridpoint islands
    for i = 1:length(LON)-3
        for j = 1:length(LAT)-3
            mini_mask = ocean_mask(i:i+3,j:j+3) ;
            % only if completely encircled by land
            if (sum(ocean_mask(i:i+3,j)) + sum(ocean_mask(i:i+3,j+3)) + sum(ocean_mask(i,j+1:j+2)) + sum(ocean_mask(i+3,j+1:j+2))) == 12
            if sum(mini_mask(:))/16 < 0.15
                ocean_mask(i:i+3,j:j+3) = 0 ;
            end
            end
        end
    end
end
if FigFlag
    subplot 313
    contourf(LON,LAT,ocean_mask') ;
    colormap(gray) ; colorbar ;
    xlabel('Longitude','fontsize',14) ;
    ylabel('Latitude','fontsize',14) ;
    set(gca,'linewidth',1.5,'box','off','tickdir','out','fontsize',12)
    title(['  Ocean mask for:     Land = Water Percentage <= ',int2str(cutoff),'%  '],'fontsize',18)
end
% Save to nc file
if ncFlag
    disp(['Ocean mask file saved at ',ncAddress]) ;
    if exist(ncAddress,'file')
        delete(ncAddress) ;
    end
    nccreate(ncAddress,'ocean_mask','Dimensions',{'lon' length(LON) 'lat' length(LAT)},'Format','classic');
    ncwrite(ncAddress,'ocean_mask',ocean_mask);
end