% GOAT produce Calendar means (e.g., Daily to Monthly)
% Copyright (C) Ori Adam Jan.2012.
% edited Feb.2014 by Ori Adam
% syntax:
% >> GOAT_Calendar_Mean(InputCalendar,OutputCalendar,varargin)
% Input arguments:
% InputCalendar: Address of GOAT Calendar from which to calculate the time average. For example:  'Home/GOAT/Data/NCEP/Daily'
% OutputCalendar: Address of output GOAT Calendar. For example:  'Home/GOAT/Data/NCEP/Monthly'
% Possible Means are only for the MAT Calendars:
%  DailyX24--> Daily     or     Daily365X24--> Daily365     or  Daily360X24--> Daily360
%  DailyX8 --> Daily     or     Daily365X8 --> Daily365     or  Daily360X8 --> Daily360
%  DailyX4 --> Daily     or     Daily365X4 --> Daily365     or  Daily360X4 --> Daily360
%  DailyX8 --> Monthly   or     Daily365X8 --> Monthly      or  Daily360X8 --> Monthly
%  DailyX4 --> Monthly   or     Daily365X4 --> Monthly      or  Daily360X4 --> Monthly
%  Daily   --> Monthly   or     Daily365   --> Monthly      or  Daily360   --> Monthly
% varargin:
% 'include', {'Field1','Field2','Field3', ...}: convert only the fields in the list
% 'exclude', {'Field1','Field2','Field3', ...}: do not convert the fields in the list
% 'overwrite' , 'on'/'off' : overwrite existing data. Default = 'off'
% NOTE: if both 'include' and 'exclude' are used, fields appearing in both the 'include' and 'exclude' list will not be convereted.
% EXAMPLE1: GOAT_Calendar_Mean('Home/GOAT/Data/NCEP/Daily','Home/GOAT/Data/NCEP/Monthly','include','pr')
%           produce Monthly means only of the field 'pr' from Daily data
% EXAMPLE2: GOAT_Calendar_Mean('Home/GOAT/Data/NCEP/DailyX4','Home/GOAT/Data/NCEP/Daily','include',{'pr','ua'})
%           produce Daily means from DailyX4 only for the fields 'pr' and 'ua'
% EXAMPLE3: GOAT_Calendar_Mean('Home/GOAT/Data/NCEP/DailyX4','Home/GOAT/Data/NCEP/Monthly','exclude',{'BoerBeta','wap')
%           produce Monthly means of all fields except for 'BoerBeta' and 'wap' from DailyX4
%

function GOAT_Calendar_Mean(InputCalendar,OutputCalendar,varargin)
%
G.EnvironmentFlag = 1  ; % Requird for GOAT_Display_message
Verbosity = 1 ; % 0 | 1 | 2 levels of verbosity
Fields2Include = {'None'} ;
Fields2Exclude = {'None'} ;
OverwriteFlag  = 0 ; 
if nargin < 2
    disp 'GOAT_Calendar_Mean exited. Missing InputCalendar or OutputCalendar arguments.'
    help GOAT_Calendar_Mean
    return
end
if nargin > 2
    for j = 1:length(varargin)/2
        switch lower(varargin{j*2-1})
            case 'include'
                if ~iscell(varargin{j*2})
                    Fields2Include = varargin(j*2) ;
                else
                    Fields2Include = varargin{j*2} ;
                end
            case 'exclude'
                if ~iscell(varargin{j*2})
                    Fields2Exclude = varargin(j*2) ;
                else
                    Fields2Exclude = varargin{j*2} ;
                end
            case 'overwrite'
                if strcmpi(varargin{j*2},'on') ;
                    OverwriteFlag = 1 ; 
                elseif strcmpi(varargin{j*2},'off')
                    OverwriteFlag = 0 ;
                else
                    disp(['Unrecognized Overwrite Option: ',varargin{j*2},'.  Valid options are ''on'' or ''off'''])
                    return
                end
            otherwise
                if Verbosity
                    disp(['Unrecognized Option: ',varargin{j*2}])
                    disp 'Produce_Calendar_Mean aborted'
                end
                return
        end
    end
end
% Validate Input and Output Calendars:
[InPath,InCalendar] = fileparts(InputCalendar) ;
ValidInCalendars = {'DailyX24','DailyX8','DailyX4','Daily','Daily365X24','Daily365X8','Daily365X4','Daily365','Daily360X24','Daily360X8','Daily360X4','Daily360'} ;
if ~ismember(InCalendar,ValidInCalendars)
    disp('Invalid Input Calendar. Valid Input Calendars are:')
    disp(char(ValidInCalendars)) ;
    return
end
[OutPath,OutCalendar] = fileparts(OutputCalendar) ;
switch InCalendar
    case {'DailyX24','DailyX8','DailyX4'}
        ValidOutCalendars = {'Daily','Monthly'} ;
    case {'Daily','Daily365','Daily360'}
        ValidOutCalendars = {'Monthly'} ;
    case {'Daily365X24','Daily365X8','Daily365X4'}
        ValidOutCalendars = {'Daily365','Monthly'} ;
    case {'Daily360X24','Daily360X8','Daily360X4'}
        ValidOutCalendars = {'Daily360','Monthly'} ;
end
if ~ismember(OutCalendar,ValidOutCalendars)
    disp(['Invalid Output Calendar. Valid Output Calendars for the input Calendar ',InCalendar,' are:'])
    disp(char(ValidOutCalendars)) ;
    return
end

[YYY,X] = GOAT_ParseTimeResolution(InCalendar) ;

tic ;

if ismember('None',Fields2Include)
    % Locate Existing Fields
    Fieldsdir = dir(InputCalendar) ;
    Fields2Include = cell(1,1) ;
    NF = 0 ;
    for j = 1:length(Fieldsdir)
        if Fieldsdir(j,1).isdir && ...
                ~strcmp(Fieldsdir(j,1).name,'.') && ...
                ~strcmp(Fieldsdir(j,1).name,'..')
            NF = NF + 1 ;
            Fields2Include(NF,1) = {Fieldsdir(j,1).name} ;
        end
    end
end
GOAT_Display_message(G,'','init')
for f = 1 : length(Fields2Include)
    if sum(strcmp(Fields2Include{f},Fields2Exclude)) ; continue ; end
    yearsdir = dir(fullfile(InputCalendar,Fields2Include{f},'')) ;
    NF = 0 ;
    years = [] ;
    for j = 1:length(yearsdir)
        if yearsdir(j,1).isdir && ...
                ~strcmp(yearsdir(j,1).name,'.') && ...
                ~strcmp(yearsdir(j,1).name,'..')
            NF = NF + 1 ;
            years(NF) = round(str2double(yearsdir(j,1).name)) ;
        end
    end
    % Transfer Dims Files
    if ~exist(fullfile(OutputCalendar,cell2mat(Fields2Include(f)),'Dims.mat'),'file') ;
        if ~exist(fullfile(InputCalendar,cell2mat(Fields2Include(f)),'Dims.mat'),'file') ;
            GOAT_Display_message(G,['Warning: Missing ''Dims.mat'' file in Input field. --> ',fullfile(InputCalendar,cell2mat(Fields2Include(f)),'Dims.mat')],'keepthis')
        else
            try
                mkdir(fullfile(OutputCalendar,cell2mat(Fields2Include(f)))) ;
                copyfile(fullfile(InputCalendar,cell2mat(Fields2Include(f)),'Dims.mat'),fullfile(OutputCalendar,cell2mat(Fields2Include(f)),'Dims.mat')) ;
            catch
                if Verbosity
                    GOAT_Display_message(G,['Warning: could not produce Dims file --> ',fullfile(OutputCalendar,cell2mat(Fields2Include(f)),'Dims.mat')],'keepthis')
                end
            end
        end
    end
    if strcmp(Fields2Include(f),'BoerBeta')
        BoerBetaFlag = 1 ;
    else
        BoerBetaFlag = 0 ;
    end
    if strcmp(OutCalendar,'Monthly')
        MonthlyFlag = 1 ;
    else
        MonthlyFlag = 0 ;
    end
    for y = 1: length(years)
        switch Verbosity
            case 1
                GOAT_Display_message(G,['Field ',int2str(f),' of ',int2str(length(Fields2Include)),' - ',fullfile(OutputCalendar,cell2mat(Fields2Include(f)),num2str(years(y)),'')],'stat')  ;
            case 2
                if y == 1
                    GOAT_Display_message(G,['Field ',int2str(f),' of ',int2str(length(Fields2Include)),' - ',fullfile(OutputCalendar,cell2mat(Fields2Include(f)))],'stat') ;
                end
        end
        for m = 1 : 12
            if MonthlyFlag ;
                q = 0 ;
                if ~OverwriteFlag
                    if exist(fullfile(OutputCalendar,cell2mat(Fields2Include(f)),num2str(years(y)),[int2str(m*10000),'.mat']),'file')
                        continue
                    end
                end
            end
            mdays = GOAT_DaysInMonth(m,years(y),InCalendar) ;
            for k = 1 : mdays
                if ~MonthlyFlag ; 
                    q = 0 ; 
                    if ~OverwriteFlag
                        if exist(fullfile(OutputCalendar,cell2mat(Fields2Include(f)),num2str(years(y)),[int2str(m*10000 + k*100),'.mat']),'file')
                            continue
                        end
                    end
                end
                for h = 1:X
                    try
                        clear bata data
                        load(fullfile(InputCalendar,cell2mat(Fields2Include(f)),num2str(years(y)),[int2str(m*10000+k*100 + (h-1)*24/X),'.mat'])) ;
                        if k==1 && h==1
                            if BoerBetaFlag
                                if  MonthlyFlag
                                    datam = zeros([mdays*X,size(bata)]) ;
                                else
                                    datam = zeros([X,size(bata)]) ;
                                end
                            else
                                if MonthlyFlag
                                    datam = zeros([mdays*X,size(data)]) ;
                                else
                                    datam = zeros([X,size(data)]) ;
                                end
                            end
                        end
                        q = q+1 ;
                        if BoerBetaFlag
                            datam(q,:,:,:) = bata ;
                        else
                            datam(q,:,:,:) = data ;
                        end
                        
                    catch
                        % Nevermind, Missing days
                    end
                end
                if ~MonthlyFlag
                    if q == X
                        FileAddress = fullfile(OutputCalendar,cell2mat(Fields2Include(f)),num2str(years(y)),[int2str(m*10000 + k*100),'.mat']) ;
                        if ~exist(fileparts(FileAddress),'dir')
                            mkdir(fileparts(FileAddress)) ;
                        end
                        if BoerBetaFlag
                            bata = squeeze(single(nanmean(datam,1))) ;
                            save(FileAddress,'bata') ;
                        else
                            data = squeeze(single(nanmean(datam,1))) ;
                            save(FileAddress,'data') ;
                        end
                    else
                        disp(['Warning: missing frames in ',fullfile(InputCalendar,cell2mat(Fields2Include(f)),num2str(years(y)),[int2str(m*10000+k*100),'.mat'])])
                    end
                end
            end
            if MonthlyFlag
                if q == X*mdays
                    FileAddress = fullfile(OutputCalendar,cell2mat(Fields2Include(f)),num2str(years(y)),[int2str(m*10000),'.mat']) ;
                    if ~exist(fileparts(FileAddress),'dir')
                        mkdir(fileparts(FileAddress)) ;
                    end
                    if BoerBetaFlag
                        bata = squeeze(single(nanmean(datam,1))) ;
                        save(FileAddress,'bata') ;
                    else
                        data = squeeze(single(nanmean(datam,1))) ;
                        save(FileAddress,'data') ;
                    end
                else
                    disp(['Warning: missing frames in ',fullfile(InputCalendar,cell2mat(Fields2Include(f)),num2str(years(y)),[int2str(m*10000),'.mat'])])
                end
            end
        end
    end
end
t = toc ;
[h,m,s,TimeString] = GOAT_UTILS_seconds2time(t) ;
if Verbosity
    disp([InCalendar,' to ',OutCalendar,' conversion completed in ',TimeString]) ;
end