% GOAT_Dataset_Clim  produces a mirror dataset with climatological fields
% Copyright (C) Ori Adam, Dec.2014
% Syntax:
% >> GOAT_Dataset_Clim(DataSet_Address)
% or
% >> GOAT_Dataset_Clim(DataSet_Address,G,varargin) % G is GOAT main structure, by default derived by G = GOAT_Environment
% varargin:
% 'outputfolder' :  Address of climatology dataset. Default is  DataSet_Address_Clim 
% 'climyear'     : 1000 by default. The Artificial year of climatology data
% 'year1'        : Initial year of data used to derive climatology
% 'year2'        : Last year of data used to derive climatology
% 'fieldstoinclude': String array of physical field names to be converted to climatology (default is all fields) 
% 'fieldstoexclude': String array of physical field names not to be converted to climatology (default is all fields) 
% 'calendarstoinclude': String array of Calendars (DailyX4, Daily, Monthly, ...) to be converted. Default is all available Calendars
% 'virtualfields': String array of Virtual Field names for which a physical field climatology will be derived

function status =  GOAT_Dataset_Clim(DataSet_Address,G,varargin)
status = 0 ; % Successful/Unseccessful   1/0
if nargin == 1
    G = GOAT_Environment  ; % Initiate GOAT Environment
end
G.Settings.Default.DisplayProgressCutoff = 5000 ;
DataSet_Clim = [DataSet_Address,'_Clim']  ;
ClimYear           = '1000'                 ;
FieldsToInclude    = {'all'}                ; 
FieldsToExclude    = {'Indices','indices'} ;
FieldsToExclude0   = {'none'}               ; 
CalendarsToInclude = {'Monthly','Daily','DailyX4','Julian'} ;
VirtualFields = [] ; 
Year1 = [] ; 
Year2 = [] ; 
% Options
if nargin > 2
    for i = 1:length(varargin)/2
        switch lower(varargin{i*2-1})
            case 'outputfolder'
                DataSet_Clim = varargin{i*2} ;    
            case 'climyear'
                ClimYear = varargin{i*2} ;
            case 'year1'
                Year1 = varargin{i*2} ;    
            case 'year2'
                Year2 = varargin{i*2} ;        
            case 'fieldstoinclude'
                if iscell(varargin{i*2})
                    FieldsToInclude = varargin{i*2} ;    
                else
                    FieldsToInclude = varargin(i*2) ;    
                end
            case 'calendarstoinclude'
                if iscell(varargin{i*2})
                    CalendarsToInclude = varargin{i*2} ;    
                else
                    CalendarsToInclude = varargin(i*2) ;    
                end    
                
            case 'virtualfields'
                if iscell(varargin{i*2})
                    VirtualFields = varargin{i*2} ;    
                else
                    VirtualFields = varargin(i*2) ;    
                end    
            case 'fieldstoexclude'
                if iscell(varargin{i*2})
                    FieldsToExclude0 = varargin{i*2} ;    
                else
                    FieldsToExclude0 = varargin(i*2) ;    
                end    
        end
    end
end
if ~ismember('none',FieldsToExclude0)
    FieldsToExclude = [FieldsToExclude,FieldsToExclude0] ;
end
% Get Calendars

Calendars = dir(DataSet_Address) ;
Calendars = {Calendars([Calendars(:).isdir],1).name}' ;
Calendars = Calendars(~logical(strncmp(Calendars,'.',1))) ;
isValid = zeros(length(Calendars),1) ;
for i = 1:length(Calendars)
    for v = 1:length(CalendarsToInclude)
        if ismember(Calendars{i},CalendarsToInclude)
            isValid(i) = 1 ;
        end
    end
end
if ~sum(isValid)
    disp('GOAT_Dataset_Clim aborted because: ')
    disp(DataSet_Address)
    disp 'does not contain valid GOAT Calendars'
    return
end
Calendars = Calendars(isValid==1) ;
for c = 1:length(Calendars)
    Calendar = Calendars{c} ;
    disp(['Creating ',fullfile(DataSet_Clim,Calendar),' calendar . . . '])
    Fields = dir(fullfile(DataSet_Address,Calendar)) ;
    Fields = {Fields([Fields(:).isdir],1).name}' ;
    G = GOAT_Initialize_Dataset(G,DataSet_Address,Calendar) ; % Go through contents of Data folder and Calendar. Link existing fields, temporal span, etc.
    % Create Mirror Clim directory and copy meta data
    mkdir(fullfile(DataSet_Clim,Calendar)) ;
    if exist(fullfile(DataSet_Address,Calendar,'info.mat'),'file')
        copyfile(fullfile(DataSet_Address,Calendar,'info.mat'),fullfile(DataSet_Clim,Calendar,'info.mat')) ;
    end
    if exist(fullfile(DataSet_Address,Calendar,'Topography.mat'),'file')
        copyfile(fullfile(DataSet_Address,Calendar,'Topography.mat'),fullfile(DataSet_Clim,Calendar,'Topography.mat')) ;
    end
    if exist(fullfile(DataSet_Address,Calendar,'link_info.txt'),'file')
        copyfile(fullfile(DataSet_Address,Calendar,'link_info.txt'),fullfile(DataSet_Clim,Calendar,'link_info.txt')) ;
    end
    % Obtain Fields
    Fields = dir(fullfile(DataSet_Address,Calendar)) ;
    Fields = {Fields([Fields(:).isdir],1).name}' ;
    Fields = Fields(~logical(strncmp(Fields,'.',1))) ;
    isValid = ones(length(Fields),1) ;
    for i = 1:length(Fields)
        for v = 1:length(FieldsToExclude)
            if ~ismember('all',FieldsToInclude)
                if ~ismember(Fields{i},FieldsToInclude)
                    isValid(i) = 0 ;
                end
            end
            if ismember(Fields{i},FieldsToExclude)
                isValid(i) = 0 ;
            end
        end
    end
%     if ~ismember('all',FieldsToInclude)
%         IncludedFields = FieldsToInclude(ismember(FieldsToInclude,Fields)) ; 
%         VirtualFieldsToInclude = FieldsToInclude(~ismember(FieldsToInclude,Fields)) ;
%     end
    if sum(isValid) == 0 && isempty(VirtualFields)
        continue
    end
    Fields = Fields(isValid==1) ;
    if ~isempty(VirtualFields)
        Fields = [Fields ; VirtualFields'] ; 
    end
%     BetaFlag = 0 ; 
%     if ismember('BoerBeta',Fields)
%         BetaFlag = 1 ; 
%     end
%     if BetaFlag % This stage comes after verifying there are fields to be converted
%       disp(['Creating mirror ',fullfile(DataSet_Clim,Calendar,'BoerData'),' field . . .'])
%       G = BoerBeta2data(G,DataSet_Address,Calendar) ; % Creates a mirror BoerData field containing data instead of bata data matrices
%       try
%        Fields = [{'BoerData'},Fields] ; 
%       catch
%        Fields = [{'BoerData'};Fields] ;    
%       end
%     end
    FramesInCalendar = GOAT_getSerial(1000,1,1,0,Calendar,999) - GOAT_getSerial(999,1,1,0,Calendar,999) ;
    for f = 1:length(Fields)
        Field = Fields{f} ;
        GOAT_Display_message(G,'','init') ;        
        GOAT_Display_message(G,['Converting field ',int2str(f),'/',int2str(length(Fields)),'  ',Field,'  . . . '],'stat')
        %disp(['Converting field ',int2str(f),'/',int2str(length(Fields)),'  ',Field,'  . . . '])
        G = GOAT_Initialize_Field(G,Field)        ; % get spatial and temporal dimensions of field, display settings, topography, and file addresses. Link if necessary
        if strcmp(G.DataFieldReference,G.DataField)
            % Create Mirror Clim Field directory and copy meta data
            if exist(fullfile(DataSet_Address,Calendar,Field,'Dims.mat'),'file')
                mkdir(fullfile(DataSet_Clim,Calendar,Field,ClimYear)) ;
                copyfile(fullfile(DataSet_Address,Calendar,Field,'Dims.mat'),fullfile(DataSet_Clim,Calendar,Field,'Dims.mat')) ;
            else
                disp(['Cannot locate Dims file for ',fullfile(DataSet_Address,Calendar,Field),' !  Conversion to Climatology aborted'])
                continue ;
            end
            if exist(fullfile(DataSet_Address,Calendar,Field,'info.mat'),'file')
                copyfile(fullfile(DataSet_Address,Calendar,Field,'info.mat'),fullfile(DataSet_Clim,Calendar,Field,'info.mat')) ;
            end
            if exist(fullfile(DataSet_Address,Calendar,Field,'Topography.mat'),'file')
                copyfile(fullfile(DataSet_Address,Calendar,Field,'Topography.mat'),fullfile(DataSet_Clim,Calendar,Field,'Topography.mat')) ;
            end
            if exist(fullfile(DataSet_Address,Calendar,Field,'link_info.txt'),'file')
                copyfile(fullfile(DataSet_Address,Calendar,Field,'link_info.txt'),fullfile(DataSet_Clim,Calendar,Field,'link_info.txt')) ;
            end
        else
            mkdir(fullfile(DataSet_Clim,Calendar,Field,ClimYear)) ;
            lev = G.lev ; levlabel = G.levlabel ; lon = G.lon ; lonlabel = G.lonlabel ; lat = G.lat ; latlabel = G.latlabel ;
            FieldDescription = G.FieldDescription ; Units = G.DataFieldUnits ; VerticalOrdinate = G.VerticalOrdinate ;
            save(fullfile(DataSet_Clim,Calendar,Field,'Dims.mat'),'lev','lon','lat','levlabel','lonlabel','latlabel','FieldDescription','VerticalOrdinate','Units') ;
            if exist(fullfile(DataSet_Address,Calendar,G.DataFieldReference,'info.mat'),'file')
                copyfile(fullfile(DataSet_Address,Calendar,G.DataFieldReference,'info.mat'),fullfile(DataSet_Clim,Calendar,Field,'info.mat')) ;
            end
            if exist(fullfile(DataSet_Address,Calendar,G.DataFieldReference,'Topography.mat'),'file')
                copyfile(fullfile(DataSet_Address,Calendar,G.DataFieldReference,'Topography.mat'),fullfile(DataSet_Clim,Calendar,Field,'Topography.mat')) ;
            end
            if exist(fullfile(DataSet_Address,Calendar,G.DataFieldReference,'link_info.txt'),'file')
                copyfile(fullfile(DataSet_Address,Calendar,G.DataFieldReference,'link_info.txt'),fullfile(DataSet_Clim,Calendar,Field,'link_info.txt')) ;
            end
        end
        G.Limits.Default.Day2same   = 0 ; % 1/0 Set Date 2 same as Date 1
        G.Limits.Default.Sequential = 0 ; % 1/0 Sequential or Seasonal averaging Flag
        G.Limits.Default.ClimatologyContinuous = 0 ; % present seasonal data sequentially in Hovmoller plots
        G.Limits.Default.Anom  = 0      ; % display anomalies in Hovmoller plots
        for n = 1:FramesInCalendar
            GOAT_Display_message(G,['Converting field ',int2str(f),'/',int2str(length(Fields)),'  ',Field,'  . . . ',num2str(n/FramesInCalendar*100,'%02.0f'),' %'],'stat')
            [year1,month,day,hour] = GOAT_serial2date(G.Addresses.Default,n) ;
            if n < G.Limits.Smin
                year1 = year1 + 1 ;
            end
            if ~isempty(Year1)
                if Year1 > year1
                    year1 = Year1 ;
                end
            end
            G.Limits.S1 = GOAT_getSerial(year1,month,day,hour,Calendar,G.Limits.Year0) ;
            year2 = G.Limits.Year0 + G.Limits.LastYear - 1 ;
            G.Limits.S2 = GOAT_getSerial(year2,month,day,hour,Calendar,G.Limits.Year0) ;
            if G.Limits.S2 > G.Limits.Smax
                year2 = year2 - 1 ;
                G.Limits.S2 = GOAT_getSerial(year2,month,day,hour,Calendar,G.Limits.Year0) ;
            end
            if ~isempty(Year2)
               if Year2 < year2
                   year2 = Year2 ; 
                   G.Limits.S2 = GOAT_getSerial(year2,month,day,hour,Calendar,G.Limits.Year0) ;
               end
            end
            % No need to set spatial limits because default values span entire spatial limits
            G.Settings.NanMeanFlag = 0  ; % use mean(x) when 0 and nanmean(x) when 1
            G.Filter.Index = ones(length(G.Addresses.Default),1) ; % default index
            DateAddress = fullfile(ClimYear,[int2str(month*10000 + day*100+hour),'.mat']) ;
            if ~strcmp(Field,'BoerBeta')
               G = GOAT_Compile_data_3D(G) ;
               data = single(G.Data) ;
               save(fullfile(DataSet_Clim,Calendar,Field,DateAddress),'data') ;
            else
               G = GOAT_Compile_bata_3D(G) ; 
               bata = single(G.Bata) ;
               save(fullfile(DataSet_Clim,Calendar,Field,DateAddress),'bata') ;
            end
        end
%         if strcmp(Field,'BoerData')
%             try
%                 rmdir(fullfile(DataSet_Address,Calendar,'BoerData'),'s') ; % Remove BoerData field
%                 movefile(fullfile(DataSet_Clim,Calendar,'BoerData'),fullfile(DataSet_Clim,Calendar,'BoerBeta')) ; % Remove BoerData field
%             catch
%                 copyfile(fullfile(DataSet_Clim,Calendar,'BoerData'),fullfile(DataSet_Clim,Calendar,'BoerBeta')) ; % Remove BoerData field
%                 disp(['Error trying to remove ',fullfile(DataSet_Address,Calendar,'BoerData'),' and renaming BoerData --> BoerBeta. ']) ;
%             end
%         end
    end
end
status = 1 ; 
disp(['Climatology dataset ',DataSet_Clim,' successfuly created. '])

function G = BoerBeta2data(G,DataSet_Address,Calendar)  % Creates a mirror BoerData field containing data instead of bata data matrices
BoerBetaDir = fullfile(DataSet_Address,Calendar,'BoerBeta') ;
BoerDataDir = fullfile(DataSet_Address,Calendar,'BoerData') ;
G = GOAT_Initialize_Dataset(G,DataSet_Address,Calendar) ; % Go through contents of Data folder and Calendar. Link existing fields, temporal span, etc.
G = GOAT_Initialize_Field(G,'BoerBeta')        ; % get spatial and temporal dimensions of field, display settings, topography, and file addresses. Link if necessary
G.DataStringVec = G.Limits.Smin : G.Limits.Smax ;
if exist(fullfile(BoerBetaDir,'Dims.mat'),'file')
    mkdir(BoerDataDir) ;
    copyfile(fullfile(BoerBetaDir,'Dims.mat'),fullfile(BoerDataDir,'Dims.mat')) ;
else
    disp(['Cannot locate Dims file for ',BoerBetaDir,' !  Creating Mirror BoerData  aborted'])
    return ;
end
if exist(fullfile(BoerBetaDir,'info.mat'),'file')
    copyfile(fullfile(BoerBetaDir,'info.mat'),fullfile(BoerDataDir,'info.mat')) ;
end
if exist(fullfile(BoerBetaDir,'Topography.mat'),'file')
    copyfile(fullfile(BoerBetaDir,'Topography.mat'),fullfile(BoerDataDir,'Topography.mat')) ;
end
if exist(fullfile(BoerBetaDir,'link_info.txt'),'file')
    copyfile(fullfile(BoerBetaDir,'link_info.txt'),fullfile(BoerDataDir,'link_info.txt')) ;
end
for s = G.DataStringVec
    %[year,month,day,hour] = GOAT_serial2date(G.Addresses.Default,s) ;
    [data,G] = GOAT_load_bata(fullfile(DataSet_Address,Calendar,'BoerBeta',G.Addresses.Default{s}),G,s)  ;
    data = single(data) ; 
    [YearBoer,NameBoer,Ext] = fileparts(G.Addresses.Default{s}) ;
    SaveAddress = fullfile(BoerDataDir,YearBoer,[NameBoer,'.mat']) ;
    if ~exist(fileparts(SaveAddress),'dir')
        mkdir(fileparts(SaveAddress)) ;
    end
    save(SaveAddress,'data')
end

