% GOAT_VirtualFields Virtual Fields Function
% Copyright (C) Ori Adam Feb.2013. compatible with version 2013i or later
% Virtual Fields require the following:
% G.VirtualFields.List: the list of 'active' virtual fields
% G.VirtualFields.RequiredFieldsList: the actual fields that need to exist for the calculation of a virtual field
% G.VirtualFields.ReferenceField: a field with the same spatial and temporal dimensions as the virtual field
% G.VirtualFields.DataFieldUnits = Virtual Field Units
% G.VirtualFields.FieldDescription = Virtual Field Description
% When Mode specifies 'Load', the code for calculating the virtual field is applied.
% The code can be placed in the VirtualFields finction or in an external .m file.
% Virtual Fields that do not appear in G.VirtualFields.List or G.VirtualFields.MyList wil not be shown in the GOAT GUI
% but will be available via GOAT_Environment coding
%
% see also GOAT_VirtualField_

function [G,data] = GOAT_VirtualFields(G,Mode,Field,datafile,DataFolder,TimeField,Aidx)
% This function contains all the operations required for adding a virtual
% Field to GOAT
switch Mode
    case 'VirtualFieldsList'
        G.VirtualFields.List = {'ta_pot','Sp','N^2','Psi'} ;
        return ;
    case 'RequiredFieldsList'  % The existing fields required for the calculation of the virtual field
        switch Field
            case {'ta_pot','Sp'}
                G.VirtualFields.RequiredFieldsList = {'ta'} ;
            case {'N^2'}
                G.VirtualFields.RequiredFieldsList = {'ta','zg'} ;
            case {'Psi'}
                G.VirtualFields.RequiredFieldsList = {'va'} ;
        end
        return ;
    case 'ReferenceField' % a field with similar Dims
        switch Field
            case   {'ta_pot','N^2','Sp'}
                G.VirtualFields.ReferenceField = 'ta' ;
            case {'Psi'}
                G.VirtualFields.ReferenceField = 'va' ;
        end
        return ;
    case 'Dims'
        switch Field  % the units and field description of the virtual field
            case 'ta_pot'
                G.VirtualFields.DataFieldUnits = 'K' ;
                G.VirtualFields.FieldDescription = 'Potential Temperature' ;
            case 'N^2'
                G.VirtualFields.DataFieldUnits = '1/s' ;
                G.VirtualFields.FieldDescription = 'Brunt Vaisala Frequency' ;
            case 'Sp'
                G.VirtualFields.DataFieldUnits = 'K/Pa' ;
                G.VirtualFields.FieldDescription = 'Static Stability = -ta/ta_pot*d(ta_pot)/dp' ;
            case 'Psi'
                G.VirtualFields.DataFieldUnits = 'kg/s' ;
                G.VirtualFields.FieldDescription = 'Meridional Stream Function' ;
        end
        return ;
    case 'Load'
        switch Field
            case 'Psi'
                G.DataField = 'va' ; [data,G] = GOAT_load_data(fullfile(DataFolder,TimeField,G.DataField,datafile),G,Aidx) ;
                try
                    [bata,G] = GOAT_load_bata(fullfile(DataFolder,TimeField,'BoerBeta',datafile),G,Aidx) ;
                catch % no BoerBeta function
                    bata = ones(size(data)) ;
                end
                if G.lev(1) < G.lev(end)
                    data = (G.Consts.EarthRadius/G.Consts.GravitationalConst).*2.*pi.* ...
                        cumtrapz(double(G.P)*100,bata.*data.*G.Vectors.cosmat,3) ;
                else
                    P = flipdim(G.P,1) ;
                    data = flipdim(bata.*data,3) ;
                    data = (G.Consts.EarthRadius/G.Consts.GravitationalConst).*2.*pi.* ...
                        cumtrapz(double(P)*100,data.*G.Vectors.cosmat,3) ;
                    data = flipdim(data,3) ;
                end
                G.DataField = 'Psi' ;
            case 'ta_pot'
                G.DataField = 'ta' ; [data,G] = GOAT_load_data(fullfile(DataFolder,TimeField,G.DataField,datafile),G,Aidx) ;
                %load(fullfile(DataFolder,TimeField,'ta',datafile)) ;
                data = data.*(1000./G.Vectors.levmat).^(2/7) ;
                G.DataField = 'ta_pot' ;
            case 'Sp'
                G.DataField = 'ta' ; [data,G] = GOAT_load_data(fullfile(DataFolder,TimeField,G.DataField,datafile),G,Aidx) ;
                %load(fullfile(DataFolder,TimeField,'ta',datafile)) ;
                Tpot = data.*(1000./G.Vectors.levmat).^(2/7) ;
                dTdZ = diff(Tpot,1,3)./diff(G.Vectors.levmat*100,1,3) ;
                P2 =  single(G.lev(2:end) + G.lev(1:end-1))/2 ;
                data = -(G.Vectors.levmat/1000).^(2/7).*permute(interp1(P2,permute(dTdZ,[3 1 2]),single(G.P),'linear','extrap'),[2 3 1]) ;
                G.DataField = 'Sp' ;
            case 'N^2'
                % N^2 = g/Tpot*d(Tpot)/dz = g^2/Tpot*d(Tpot)/d(gph)
                G.DataField = 'ta' ; [data,G] = GOAT_load_data(fullfile(DataFolder,TimeField,G.DataField,datafile),G,Aidx) ;
                %load(fullfile(DataFolder,TimeField,'ta',datafile)) ;
                Tpot = data.*(1000./G.Vectorts.Pmat).^(2/7) ;
                G.DataField = 'zg' ; [data,G] = GOAT_load_data(fullfile(DataFolder,TimeField,G.DataField,datafile),G,Aidx) ;
                %load(fullfile(DataFolder,TimeField,'zg',datafile)) ;
                dTdZ = diff(Tpot,1,3)./diff(data,1,3) ;
                P2 = single(G.lev(2:length(G.P)) + G.lev(1:length(G.P)-1))/2 ;
                data = G.Consts.GravitationalConst./Tpot.*permute(interp1(P2,permute(dTdZ,[3 1 2]),single(G.P),'linear','extrap'),[2 3 1]) ;
                G.DataField = 'Sp' ;
        end
end

return
