% Produce composite from index

function G = GOAT_Script_IndexComposite(G)
% Copyright (C) Ori Adam 2012. Compatible with version 2013h and onwards
% This Tool produces the following:
% 1. select an index
% 2. select temporal and spatial limits
% 3. using the default field specifications (Field, Projection, Filter), a composite is generated.
% 4. Output:
%           1. Mean Composite Figure
%           2. Anomaly Composite 
%           3. Climatology figure
%           4. Composite Figs with Significance Shading
%           5. *.mat file with Data and Significance       
try
[fid,fidf] = uigetfile ;
[fidpath,fidname,fidext] = fileparts(fid) ; 
load([fidf,fid]) ;
catch
disp 'could not load index' ; 
return ; 
end
G.Limits.Default.Day2same = 0 ;
G = GOAT_Limits(G) ;
% Create Field Data Series
G = GOAT_Compile_data_Climatology(G) ; % Calculate Mean Field
G.Limits.Default.DataSeriesAnomFlag = 1 ; 
G = GOAT_Compile_data_Series(G) ;
qF = size(G.DataSeries,1) ;
DataSeries     = reshape(G.DataSeries,qF,size(G.DataSeries,2)*size(G.DataSeries,3)) ;
DataSeriesAnom = reshape(G.DataSeriesAnom,qF,size(G.DataSeries,2)*size(G.DataSeries,3)) ;
SeriesIndex = Index(G.DataStringVec) ;
% Calculate Composites and Significances
Composite.DataClimatology = squeeze(mean(G.DataSeries(:,:,:),1)) ;
Composite.Data     = squeeze(mean(G.DataSeries(SeriesIndex==1,:,:),1)) ;
Composite.DataAnom = squeeze(mean(G.DataSeriesAnom(SeriesIndex==1,:,:),1)) ;
[s,p] = ttest2(DataSeries(SeriesIndex==1,:),DataSeries) ; 
Composite.pvalue = reshape(p,size(G.DataSeries,2),size(G.DataSeries,3)) ;
[s,p] = ttest2(DataSeriesAnom(SeriesIndex==1,:),DataSeriesAnom) ; 
Composite.pvalueAnom = reshape(p,size(G.DataSeries,2),size(G.DataSeries,3)) ;
Composite.x = G.X ; 
Composite.y = G.Y ;
try
   Composite.x = G.x ; 
   Composite.y = G.y ;
end
G.MeanFig = figure('name','Field Index Composite','position',G.pos) ;
set(0,'CurrentFigure',G.MeanFig)      ; % New Figure Without GUI Properties
G = plot_MAP(G,G,Composite.DataClimatology) ;
title(['$$\textrm{Climatology ',G.DataField,'}$$'],'interpreter','latex','fontsize',13) ;
G.CompositeFig = figure('name','Field Index Composite','position',G.pos) ;
set(0,'CurrentFigure',G.CompositeFig)      ; % New Figure Without GUI Properties
G = plot_MAP(G,G,Composite.Data) ;
title(['$$\textrm{Index ',G.DataField,'}$$'],'interpreter','latex','fontsize',13) ;
G.CompositeSigFig = figure('name','Field Index Composite','position',G.pos) ;
set(0,'CurrentFigure',G.CompositeSigFig)      ; % New Figure Without GUI Properties
G = GOAT_SignificancePlot(G,Composite.Data,Composite.pvalue,.05) ;
title(['$$\textrm{Index Significance ',G.DataField,'}$$'],'interpreter','latex','fontsize',13) ;
% Anomaly
G.CompositeAnomFig = figure('name','Field Index Anomaly Composite','position',G.pos) ;
set(0,'CurrentFigure',G.CompositeAnomFig)      ; % New Figure Without GUI Properties
G = plot_MAP(G,G,Composite.DataAnom) ;
title(['$$\textrm{Index Anomaly ',G.DataField,'}$$'],'interpreter','latex','fontsize',13) ;
G.CompositeSigAnomFig = figure('name','Field Index Anomaly Composite','position',G.pos) ;
set(0,'CurrentFigure',G.CompositeSigAnomFig)      ; % New Figure Without GUI Properties
G = GOAT_SignificancePlot(G,Composite.DataAnom,Composite.pvalueAnom,.05) ;
title(['$$\textrm{Index Anomaly Significance ',G.DataField,'}$$'],'interpreter','latex','fontsize',13) ;
ExportFolder = fullfile('..','Export',['IndComp_',G.DataField,'_',fidname],'') ;  
mkdir(ExportFolder)   ; 
save( fullfile(ExportFolder,'Data.mat'),'Composite') ; 
% Mean
saveas(G.MeanFig,fullfile(ExportFolder,'Climatology.fig'))   ; 
set(G.MeanFig,'PaperPositionMode','auto','papersize',G.Saving.PapersizeGUI)          ;
print(G.MeanFig,'-dpdf','-r70',fullfile(ExportFolder,'Climatology')) ;
% Composite
saveas(G.CompositeFig,fullfile(ExportFolder,'Composite.fig'))   ; 
set(G.CompositeFig,'PaperPositionMode','auto','papersize',G.Saving.PapersizeGUI)          ;
print(G.CompositeFig,'-dpdf','-r70',fullfile(ExportFolder,'Composite')) ;
% Composite Significance
saveas(G.CompositeSigFig,fullfile(ExportFolder,'CompositeSig.fig'))   ; 
set(G.CompositeSigFig,'PaperPositionMode','auto','papersize',G.Saving.PapersizeGUI)          ;
print(G.CompositeSigFig,'-dpdf','-r70',fullfile(ExportFolder,'CompositeSig')) ;
% Composite Anom
saveas(G.CompositeAnomFig,fullfile(ExportFolder,'CompositeAnom.fig'))   ; 
set(G.CompositeAnomFig,'PaperPositionMode','auto','papersize',G.Saving.PapersizeGUI)          ;
print(G.CompositeAnomFig,'-dpdf','-r70',fullfile(ExportFolder,'CompositeAnom')) ;
% Composite Anom Significance
saveas(G.CompositeSigAnomFig,fullfile(ExportFolder,'CompositeAnomSig.fig'))   ; 
set(G.CompositeSigAnomFig,'PaperPositionMode','auto','papersize',G.Saving.PapersizeGUI)          ;
print(G.CompositeSigAnomFig,'-dpdf','-r70',fullfile(ExportFolder,'CompositeAnomSig')) ;

set(0,'CurrentFigure',G.GOAT_Fig)                        ; % Override existing GUI Figure
G.Plotting.GUI_PlotFlag = 1                              ; % Informs GOAT_plotUpdate that the plotting is not in GUI format
G.message = dateMessage(G) ;
set(G.h_message,'string',['Composite Complete.  ',G.message]) ; % Update Message
G.Limits.Default.DataSeriesAnomFlag = 0 ; 
return ;
% plot SOM
function G = plot_MAP(G,D,Data,SigData)
try
    if D.SOM.DecimationTypeIdx > 1
        X    = DecimationFunc(D.X,D.SOM.DecimationYIdx,D.SOM.DecimationXIdx,D.SOM.DecimationTypeIdx) ;
        Y    = DecimationFunc(D.Y,D.SOM.DecimationYIdx,D.SOM.DecimationXIdx,D.SOM.DecimationTypeIdx) ;
    else
        X = D.X ;
        Y = D.Y ;
    end
    % Use Field1 struct
    Lat1 = max([D.SOM.Field1.lat(D.SOM.Field1.Lat1),D.SOM.Field1.lat(D.SOM.Field1.Lat2)]) ;
    Lat2 = max([D.SOM.Field1.lat(D.SOM.Field1.Lat1),D.SOM.Field1.lat(D.SOM.Field1.Lat2)]) ;
    Lon1 = min(D.x) ; %min([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
    Lon2 = max(D.x) ; %max([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
    P1   = min([D.SOM.Field1.P(D.SOM.Field1.P1),D.SOM.Field1.P(D.SOM.Field1.P2)]) ;
    P2   = max([D.SOM.Field1.P(D.SOM.Field1.P1),D.SOM.Field1.P(D.SOM.Field1.P2)]) ;
catch % Not a SOM data
    Lat1 = min([G.lat(G.Limits.Default.Lat1),G.lat(G.Limits.Default.Lat2)]) ;
    Lat2 = max([G.lat(G.Limits.Default.Lat1),G.lat(G.Limits.Default.Lat2)]) ;
    try
        Lon1 = min(G.x) ; % min([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
        Lon2 = max(G.x) ; % max([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
    catch
        Lon1 = min([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
        Lon2 = max([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
    end
    P1   = min([G.P(G.Limits.Default.Lev1),G.P(G.Limits.Default.Lev2)]) ;
    P2   = max([G.P(G.Limits.Default.Lev1),G.P(G.Limits.Default.Lev2)]) ;
    X = G.X ; 
    Y = G.Y ; 
end
try
    Projection = D.SOM.Field1.Projection ; 
catch
    Projection = G.ProjectionType ; 
end

try
    ContourIdx = G.Settings.ContourIdx ;
catch
    ContourIdx = 1 ;
end
try
    LineWidth      = G.Settings.LineWidth      ;
    CoastLineWidth = G.Settings.CoastLineWidth ;
catch
    LineWidth      = 1 ;
    CoastLineWidth = 1 ;
end
try
    ColorMapStr = cell2mat(G.Settings.ColorMap(G.Settings.ColorMapIdx)) ;
    switch ColorMapStr
        case 'ceJulien'
            ColorMapDef = G.Settings.ColorMaps.ceJulien ;
        case 'BlueWhiteRed'
            ColorMapDef = G.Settings.ColorMaps.BlueWhiteRed ;
        case 'Rose'
            ColorMapDef = G.Settings.ColorMaps.Rose ;
        otherwise
            ColorMapDef = ColorMapStr ;
    end
catch
    ColorMapDef = 'jet' ;
end
switch Projection
    case {1,2} % Long-Lat
        try
            if nargin == 4  && sum(sum(isnan(SigData)))
                area(G.x,G.x*0 + 1000,'FaceColor',[.85 .85 .85],'EdgeColor','black') ;
                hold on ;
                [C,h] =  contourf(X,Y,Data'.*SigData') ; hold on
                [Cs,s] =  contour(X,Y,Data') ; hold on
                if LineWidth
                    try  set(s,'linewidth',LineWidth) ; end
                end
                LineWidth = 0 ;
            else
                switch ContourIdx
                    case 1
                        [C,h] =  contourf(X,Y,Data') ;
                    case 2
                        [C,h] =  contour(X,Y,Data') ;
                    case 3
                        h =  surf(double(X),double(Y),double(Data')) ;
                    case 4
                        h =  surfc(double(X),double(Y),double(Data')) ;
                    case 5
                        h =  mesh(double(X),double(Y),double(Data')) ;
                    case 6
                        h =  meshc(double(X),double(Y),double(Data')) ;
                    case 7
                        h =  meshz(double(X),double(Y),double(Data')) ;
                    case 8
                        h =  imagesc([Lon1 Lon2],[Lat1 Lat2],double(Data')) ;
                end
            end
            colormap(ColorMapDef) ; colorbar('fontname',G.GUI.Fonts.gcaFont) ;hold on
            %clabel(C,h,'fontsize',13) ;
            if LineWidth > 0
                try  set(h,'linewidth',LineWidth) ; end
            else
                try  set(h,'linestyle','none') ; end
            end
            xlabel('$$\textrm{Longitude}$$','Fontsize',13,'interpreter','latex') ;
            ylabel('$$\textrm{Latitude}$$','Fontsize',13,'interpreter','latex') ;
            if G.ProjectionType == 1
                plot(G.topo.landcoverlon,G.topo.landcoverlat,'k',G.topo.landcoverlon+360,G.topo.landcoverlat,'k','Linewidth',CoastLineWidth) ;
            end
        catch
            % Nevermind, Error
        end
        hold off
        xlim([Lon1 Lon2])   ;
        ylim([Lat1 Lat2])   ;
        
    case 3 % Lat-Pressure
        try
            if G.Limits.Default.sameP || P2 == P1
                if LineWidth == 0 ; LineWidth = 1 ; end
                plot(X,Data,'k','linewidth',LineWidth) ;
                xlabel('$$\textrm{Latitude}$$','Fontsize',13,'interpreter','latex') ;
                ylabel(['$$\textrm{',G.DataField,'}$$'],'Fontsize',13,'interpreter','latex') ;
                xlim([Lat1 Lat2])   ;
            else
                if nargin == 4  && sum(sum(isnan(SigData)))
                    area(G.x,G.x*0 + 1000,'FaceColor',[.85 .85 .85],'EdgeColor','black') ;
                    hold on ;
                    [C,h] =  contourf(X,Y,Data'.*SigData') ; hold on
                    [Cs,s] =  contour(X,Y,Data') ; hold on
                    if LineWidth
                        try  set(s,'linewidth',LineWidth) ; end
                    end
                    LineWidth = 0 ;
                else
                    switch ContourIdx
                        case 1
                            [C,h] =  contourf(X,Y,Data') ;
                        case 2
                            [C,h] =  contour(X,Y,Data') ;
                        case 3
                            h =  surf(double(X),double(Y),double(Data')) ;
                        case 4
                            h =  surfc(double(X),double(Y),double(Data')) ;
                        case 5
                            h =  mesh(double(X),double(Y),double(Data')) ;
                        case 6
                            h =  meshc(double(X),double(Y),double(Data')) ;
                        case 7
                            h =  meshz(double(X),double(Y),double(Data')) ;
                        case 8
                            h =  imagesc([Lat1 Lat2],[P1 P2],double(Data')) ;
                    end
                end
                colormap(ColorMapDef) ; colorbar('fontname',G.GUI.Fonts.gcaFont) ;
                if LineWidth > 0
                    try  set(h,'linewidth',LineWidth) ; end
                else
                    try  set(h,'linestyle','none') ; end
                end
                axis ij
                xlabel('$$\textrm{Latitude}$$','Fontsize',13,'interpreter','latex') ;
                ylabel('$$\textrm{Level}$$','Fontsize',13,'interpreter','latex') ;
                ylim([P1 P2])
                xlim([Lat1 Lat2])   ;
            end
        catch
            % Nevermind, no Data
        end
    case 4 % Lon-Pressure
        try
            if G.Limits.Default.sameP
                if LineWidth == 0 ; LineWidth = 1 ; end
                plot(X,Data,'k','linewidth',LineWidth) ;
                xlabel('$$\textrm{Longitude}$$','Fontsize',13,'interpreter','latex') ;
                ylabel(['$$\textrm{',G.DataField,'}$$'],'Fontsize',13,'interpreter','latex') ;
                xlim([Lon1 Lon2])   ;
            elseif Lon1 == Lon2
                if LineWidth == 0 ; LineWidth = 1 ; end
                plot(Y,Data,'k','linewidth',LineWidth) ;
                xlabel('$$\textrm{Level}$$','Fontsize',13,'interpreter','latex') ;
                ylabel(['$$\textrm{',G.DataField,'}$$'],'Fontsize',13,'interpreter','latex') ;
                xlim([P1 P2])   ;
            else
                if nargin == 4  && sum(sum(isnan(SigData)))
                    area(G.x,G.x*0 + 1000,'FaceColor',[.85 .85 .85],'EdgeColor','black') ;
                    hold on ;
                    [C,h] =  contourf(X,Y,Data'.*SigData') ; hold on
                    [Cs,s] =  contour(X,Y,Data') ; hold on
                    if LineWidth
                        try  set(s,'linewidth',LineWidth) ; end
                    end
                    LineWidth = 0 ;
                else
                    switch ContourIdx
                        case 1
                            [C,h] =  contourf(X,Y,Data') ;
                        case 2
                            [C,h] =  contour(X,Y,Data') ;
                        case 3
                            h =  surf(double(X),double(Y),double(Data')) ;
                        case 4
                            h =  surfc(double(X),double(Y),double(Data')) ;
                        case 5
                            h =  mesh(double(X),double(Y),double(Data')) ;
                        case 6
                            h =  meshc(double(X),double(Y),double(Data')) ;
                        case 7
                            h =  meshz(double(X),double(Y),double(Data')) ;
                        case 8
                            h =  imagesc([Lon1 Lon2],[P1 P2],double(Data')) ;
                    end
                end
                colormap(ColorMapDef) ; colorbar('fontname',G.GUI.Fonts.gcaFont) ;
                if LineWidth > 0
                    try  set(h,'linewidth',LineWidth) ; end
                else
                    try  set(h,'linestyle','none') ; end
                end
                axis ij
                xlabel('$$\textrm{Longitude}$$','Fontsize',13,'interpreter','latex') ;
                ylabel('$$\textrm{Level}$$','Fontsize',13,'interpreter','latex') ;
                ylim([P1 P2])
                xlim([Lon1 Lon2])   ;
            end
        catch
            % Nevermind, no Data
        end
        
end
% Axis Configurations
try
    if G.Settings.XgridFlag
        set(gca,'XGrid','on') ;
    end
    if G.Settings.YgridFlag
        set(gca,'YGrid','on') ;
    end
end
try
    if G.Settings.ContourLevelStep
        if  G.Settings.ContourLevelStep > (max(max(Data)) - min(min(Data)))/100 ;
            set(h,'levelstep',G.Settings.ContourLevelStep) ;
        end
    end
catch
    % Nevermind
end
try
    if G.Settings.FixZLimitsFlag || G.Settings.SymZLimitsFlag
        if G.Settings.SymZLimitsFlag
            MinZ = min(min(Data));
            MaxZ = max(max(Data));
            AbsMax = max([abs(MinZ) , abs(MaxZ)]) ;
            G.Settings.ZlimitMin = - AbsMax ;
            G.Settings.ZlimitMax =   AbsMax ;
        end
        caxis([G.Settings.ZlimitMin G.Settings.ZlimitMax]) ;
        if ~G.Settings.SymZLimitsFlag
            colorbar ;
        else
            MinZ = min(min(Data));
            MaxZ = max(max(Data));
            colorbar('YLim',[MinZ MaxZ],'fontname',G.GUI.Fonts.gcaFont) ;
        end
    end
end
set(gca,'fontsize',12,'fontname',G.GUI.Fonts.gcaFont) ;
hold off ;
% Decimation Function
function Data = DecimationFunc(data,DecX,DecY,Type)
if DecX == 1 && DecY == 1
    Type = 1 ;
end
switch Type
    case 1 % None
        Data = data ;
    case 2 % Decimate
        S = size(data) ;
        if DecX == 1
            Data = zeros(S(1),ceil(S(2)/DecY)) ;
            for i = 1:S(1)
                Data(i,:) = decimate(double(data(i,:)),DecY) ;
            end
        elseif DecY == 1
            Data = zeros(ceil(S(1)/DecX),S(2)) ;
            for i = 1:S(2)
                Data(:,i) = decimate(double(data(:,i)),DecX) ;
            end
        else
            Data1 = zeros(S(1),ceil(S(2)/DecY))           ;
            Data = zeros(ceil(S(1)/DecX),ceil(S(2)/DecY)) ;
            for i = 1:S(1)
                Data1(i,:) = decimate(double(data(i,:)),DecY) ;
            end
            for i = 1:ceil(S(2)/DecY)
                Data(:,i) = decimate(Data1(:,i),DecX) ;
            end
        end
    case 3 % DownSample
        if DecX == 1
            Data = downsample(data',DecY)' ;
        elseif DecY == 1
            Data = downsample(data,DecX) ;
        else
            Data = downsample(data',DecY)' ;
            Data = downsample(Data,DecX) ;
        end
end




